
local shader
local start

function love.load()
	love.window.setMode(1600, 900, { resizable = false, vsync = true, borderless = false })

	local vert = love.filesystem.read("shader.vert")
	local frag = love.filesystem.read("shader.frag")
	shader = love.graphics.newShader(vert, frag)
	start = love.timer.getTime()
end

function love.update(dt)
	love.window.setTitle(love.timer.getFPS())
	if love.keyboard.isDown("escape") then
		love.event.quit()
	end
end

function generateMesh(bendFirstNormal)
	local MeshAttributes = {
	    {"VertexPosition", "float", 3},
	    {"VertexTexCoord", "float", 2},
	    {"VertexNormal", "float", 3},
	    {"VertexColor", "byte", 4},
	}

	local Vertices = {
        {
            0, -3, 0,
            0, -3,
            bendFirstNormal and 0.0 or 0.0, bendFirstNormal and 0.4 or 0.0, bendFirstNormal and 0.4 or 1.0,
            1, 1, 1, 1
        },
        {
            -3, 0, 0,
            -3, 0,
            0.0, 0.0, 1.0,
            1, 1, 1, 1
        },
        {
            2, 3, 0,
            2, 3,
            0.0, 0.0, 1.0,
            1, 1, 1, 1
        }
    }

	return love.graphics.newMesh(MeshAttributes, Vertices, "triangles", "static")
end

function love.draw()
	love.graphics.clear()
	love.graphics.setShader(shader)

	local t = love.timer.getTime() - start

	shader:send("World", { 1, 0, 0, 0,
                           0, 1, 0, 0,
                           0, 0, 1, 0,
                           0, 0, 0, 1 })
    shader:send("DirLights[0].AmbientIntensity", 0.1)
    shader:send("DirLights[0].DiffuseIntensity", 0.8)
    shader:send("DirLights[0].Color", { 1, 1, 1 })
    shader:send("DirLights[0].Direction", { math.sin(t), math.cos(t), -1 })
    shader:send("MaterialDiffuseColor",  { 0.6, 0.7, 0.8, 1})
    shader:send("MaterialAmbientColor", { 0.1, 0.1, 0.0, 1})
    shader:send("MaterialSpecularColor", { 0.1, 0.5, 1.1, 1})
    shader:send("CameraLocalPos", { 0.0, 0.0, 1.0 })

	local mesh1 = generateMesh(false)
    love.graphics.draw(mesh1, 500, 450, 0, 100, 100)

    local mesh2 = generateMesh(true)
    love.graphics.draw(mesh2, 1200, 450, 0, 100, 100)

    love.graphics.setShader()
    love.graphics.print("All 3 normals are equal", 500, 120, 0, 1, 1)
    love.graphics.print("Top corner normal is bend upwards", 1200, 120, 0, 1, 1)
end
