// -- -- -- -- -- -- -- -- -- -- -- -- -- -- //
// Fragment Shader 
// -- -- -- -- -- -- -- -- -- -- -- -- -- -- //

struct BaseLight {
    vec3 Color;
    float AmbientIntensity;
};

struct DirLight {
    vec3 Color;
    vec3 Direction;
    float AmbientIntensity;
    float DiffuseIntensity;
};

// Light Arrays
extern BaseLight BaseLights[1];
extern DirLight DirLights[1];

// Material
extern vec3 MaterialAmbientColor;
extern vec3 MaterialDiffuseColor;
extern vec3 MaterialSpecularColor;

extern vec3 CameraLocalPos;

varying vec3 Normal;
varying vec3 LocalPos;

vec4 effect(vec4 Color, Image image, vec2 uvs, vec2 screen_coords){
    // Interpolation: return VaryingColor;

    BaseLight gLight = BaseLights[0];
    DirLight gLight2 = DirLights[0];

    //vec4 LightColor = gLight.Color * gLight.AmbientIntensity;
    vec3 LocalAmbientColor = gLight2.Color * gLight2.AmbientIntensity * MaterialAmbientColor;

    float DiffuseFactor = dot(normalize(Normal), -gLight2.Direction);
    vec3 LocalDiffuseColor = vec3(0, 0, 0);
    vec3 LocalSpecularColor = vec3(0, 0, 0);

    if (DiffuseFactor > 0){
        LocalDiffuseColor = gLight2.Color * gLight2.DiffuseIntensity * MaterialDiffuseColor * DiffuseFactor;

        vec3 PixelToCamera = normalize(CameraLocalPos - LocalPos);
        vec3 LightReflect = normalize(reflect(gLight2.Direction, Normal));
        float SpecularFactor = dot(PixelToCamera, LightReflect);

        if (SpecularFactor > 0) {
            // texture is grayscale
            //float SpecularExponent = Texel(SpecularTexture, uvs).r * 255.0;
            SpecularFactor = pow(SpecularFactor, 3.0);
            LocalSpecularColor = gLight2.Color * MaterialSpecularColor * SpecularFactor;
        }
    }

    return Texel(image, uvs) * clamp(vec4(LocalAmbientColor + LocalDiffuseColor + LocalSpecularColor, 1.0f), 0, 1);
}
